/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package apes;

import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.geom.Shape;
import org.newdawn.slick.geom.Transform;
import org.newdawn.slick.geom.Vector2f;

/**
 * An <code>ImageObject</code> extended by the methods specified in <code>ICollidableObject</code>.
 *
 * @author sam
 */
public class CollidableImageObject extends ImageObject implements ICollidableObject {

    private Shape collisionShape;

    /**
     * creates a new collidable image object.
     *
     * @param name
     * @param image
     * @param position
     * @param collisionShape
     */
    public CollidableImageObject(String name, Image image, Vector2f position, Shape collisionShape) {
        super(name, image, position);
        this.collisionShape = collisionShape;
    }

    /**
     * sets the position of this object.
     *
     * @param position
     */
    @Override
    public void setPosition(Vector2f position) {
        super.setPosition(position);
    }

    /**
     * returns the general collision shape of this object.
     *
     * @return
     */
    @Override
    public Shape getNormalCollisionShape() {
        return collisionShape;
    }

    /**
     * returns the collision shape of this object moved to its actual position.
     *
     * @return
     */
    @Override
    public Shape getCollisionShape() {
        return collisionShape.transform(Transform.createTranslateTransform(position.x, position.y));
    }

    /**
     * draws the <code>image</code> on the screen with its top left corner
     * at the given x/y coordinates.
     * If debug is enabled it will also draw its collision shape.
     *
     * @param graphics
     */
    @Override
    public void render(Graphics graphics) {
        image.draw(position.x, position.y);

        if (Settings.DEBUG) {
            graphics.draw(getCollisionShape());
        }
    }

    /**
     * returns true if the collision shape of this object intersects with the
     * collision shape of the given object.
     * It uses the shapes provided by the {@link #getCollisionShape() getCollisionShape} method.
     *
     * @param collidable
     * @return
     */
    @Override
    public boolean isCollidingWith(ICollidableObject collidable) {
        return this.getCollisionShape().intersects(collidable.getCollisionShape());
    }
}