/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package apes;

import java.awt.Font;
import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.UnicodeFont;
import org.newdawn.slick.geom.Circle;
import org.newdawn.slick.geom.Vector2f;
import org.newdawn.slick.state.BasicGameState;
import org.newdawn.slick.state.StateBasedGame;

/**
 * @author sam
 */
public class GameState extends BasicGameState {

    private boolean isPowerUp = false;

    private static float MINVELOCITY = 0.0f;
    private static float MINBETA = 0.0f;
    private static float MAXVELOCITY = 150.0f;
    private static float MAXBETA = 90f;

    private float beta = MINBETA; // agnle object was thrown in
    private float velocity = MINVELOCITY; // velocity the object was thrown with
    private float time = 0.0f; // time since throw accoured
    private float timeEnemy = 0.0f; // time since throw accoured
    private static float g = 9.8f; // gravity

    private static int MAXDELTACOUNTER = 20;
    int deltaCounter = MAXDELTACOUNTER; // time to wait till next move update
    int deltaCounterEnemy = MAXDELTACOUNTER;

    private UnicodeFont unicodeFont;
    
    public BlockMap map;
    private IEnemyAI enemyAI;

    private enum STATES {

        START_GAME_STATE, GET_INPUT_STATE, MOVING_PIECE_STATE,
        PAUSE_GAME_STATE, GAME_OVER_STATE, LEVEL_COMPLETE_STATE
    }
    private STATES currentState = null;
    private STATES statebeforpause = null;
    private int stateID = -1;

    private Image pause = null;
    private Image won = null;
    private Image gameOver = null;

    private Menu pauseMenu;

    private String level;

    private SettingsLevel settingsLevel;
    private int lvlPlaying = -1;

    private EnemyAIChooser aichooser;

    public GameState(int stateID, String lvl) {
        this.stateID = stateID;
        this.level = lvl;
    }

    @Override
    public int getID() {
        return this.stateID;
    }

    public void setLevel(int lvlPlaying) {
        this.lvlPlaying = lvlPlaying;
    }

    public void setSettingsLevel(SettingsLevel settingsLevel) {
        this.settingsLevel = settingsLevel;
    }

    /**
     *
     * @param level path to level map
     */
    public void setLevel(String level) {
        this.level = level;
    }

    @Override
    public void init(GameContainer gc, StateBasedGame sbg) throws SlickException {

        beta = MINBETA;
        velocity = MINVELOCITY;

        map = new BlockMap(level);

        currentState = STATES.GET_INPUT_STATE;

        unicodeFont = new UnicodeFont(new Font(null, stateID, 15));

        gameOver = new Image("/data/options/gameOver.png");
        won = new Image("/data/options/levelComplete.png");
        
        pause = new Image("/data/options/optionsPause.png");

        // init pause menu
        Image continueOpt = new Image("/data/options/optionsContinue.png");
        Image toMain = new Image("/data/options/optionsToMain.png");

        float centerXPosition = (gc.getScreenWidth() - Math.max(toMain.getWidth(), continueOpt.getWidth())) / 2;
        float yspaceBetweenOptions = toMain.getHeight() / 2;
        float yPositionOfTopOption = gc.getScreenHeight() / 1.8f;
        pauseMenu = new Menu(centerXPosition, yPositionOfTopOption, yspaceBetweenOptions);
        pauseMenu.addItem(continueOpt, new MenuActionActivated() {
            @Override
            public void execute(GameContainer gc, StateBasedGame sbg, MenuItem item) throws SlickException {
                currentState = statebeforpause;
            }
        });
        pauseMenu.addItem(toMain, new MenuActionActivated() {
            @Override
            public void execute(GameContainer gc, StateBasedGame sbg, MenuItem item) throws SlickException {
                sbg.enterState(Game.MAINMENUSTAT);
            }
        });

        aichooser = new EnemyAIChooser(map);
        enemyAI = aichooser.getAI();
    }

    @Override
    public void render(GameContainer gc, StateBasedGame sbg, Graphics grphcs) throws SlickException {

        // draw map, player, banana on screen
        map.getTmap().render(0, 0);
        map.getEnemy().render(grphcs);
        map.getApe().render(grphcs);
        map.getBanana().render(grphcs);
        map.getBanana2().render(grphcs);

        // draw info about velocity, etc
        String vel = "velocity: " + ((int) velocity);
        String ang = "angle: " + ((int) beta);
        int angPos = gc.getWidth() - unicodeFont.getWidth(ang);
        grphcs.drawString(ang, gc.getWidth() - unicodeFont.getWidth(ang) - 20, unicodeFont.getHeight(ang));
        grphcs.drawString(vel, angPos - unicodeFont.getWidth(vel) - 60, unicodeFont.getHeight(vel));

        // draw state specific things
        switch (currentState) {
            case PAUSE_GAME_STATE:
                pause.drawCentered(gc.getScreenWidth() / 2, gc.getScreenHeight() / 2);
                pauseMenu.render(grphcs);
                break;
            case GAME_OVER_STATE:
                gameOver.drawCentered(gc.getScreenWidth() / 2, gc.getScreenHeight() / 2);
                break;
            case LEVEL_COMPLETE_STATE:
                won.drawCentered(gc.getScreenWidth() / 2, gc.getScreenHeight() / 2);
                break;
        }

        if (Settings.DEBUG || (Settings.MOUSEENABLED && !map.getBanana().isMoving())) {
            drawFlightPattern(gc, sbg, grphcs);
        }

        if (Settings.DEBUG) {
            try {
                Vector2f positionBeforSpawn2 = map.getBanana2().getPositionBeforSpawn();
                Circle c2 = new Circle(positionBeforSpawn2.x, positionBeforSpawn2.y, 20);
                Color color = grphcs.getColor();
                grphcs.setColor(Color.red);
                grphcs.draw(c2);
                grphcs.setColor(color);
            } catch (Exception e) {
              //  positionBeforSpawn2 may be null
            }

            try {
                Vector2f positionBeforSpawn = map.getBanana().getPositionBeforSpawn();
                Circle c = new Circle(positionBeforSpawn.x, positionBeforSpawn.y, 20);
                Color color = grphcs.getColor();
                grphcs.setColor(Color.red);
                grphcs.draw(c);
                grphcs.setColor(color);
            } catch (Exception e) {
              //  positionBeforSpawn2 may be null
            }

            try {
                grphcs.draw(((EnemyAIMedium) enemyAI).hitArea);
            } catch (Exception e) {
              
            }            

        // draw collision polygons on screen
            for (int i = 0; i < map.getEntities().size(); i++) {
                Block entity1 = (Block) map.getEntities().get(i);
                grphcs.draw(entity1.poly);
            }
            String state = currentState.name();
            grphcs.drawString(state, 200, 20);
        }
    }

    @Override
    public void update(GameContainer gc, StateBasedGame sbg, int i) throws SlickException {
        
        switch (currentState) {
            case START_GAME_STATE:
                currentState = STATES.GET_INPUT_STATE;
                break;
            case GET_INPUT_STATE:
                applyUserInput(gc, sbg, i);
                letAIMove(gc, sbg, i);
                break;
            case MOVING_PIECE_STATE:
                calculateMovement(gc, sbg, i);
                letAIMove(gc, sbg, i);
                break;
            case PAUSE_GAME_STATE:
                pauseMenu.update(gc, sbg, i);
                isEndPause(gc, sbg, i);
                break;
            case GAME_OVER_STATE:
                if (gc.getInput().isKeyPressed(Input.KEY_ENTER) || gc.getInput().isKeyPressed(Input.KEY_ESCAPE)) {
                    sbg.enterState(Game.MAINMENUSTAT);
                }
                break;
            case LEVEL_COMPLETE_STATE:
                if (gc.getInput().isKeyPressed(Input.KEY_ENTER) || gc.getInput().isKeyPressed(Input.KEY_ESCAPE)) {
                    settingsLevel.openLevel(lvlPlaying + 1);
                    sbg.enterState(Game.MAINMENUSTAT);
                }
                break;
        }
    }

    private void letAIMove(GameContainer gc, StateBasedGame sbg, int i) {

        if (map.getBanana2().isMoving()) {
            calculateEnemyMovement(gc, sbg, i);
        } else {
            timeEnemy = 0.0f;
            enemyAI.executeNextMove(timeEnemy, gc);
        }

    }
    
    private void drawFlightPattern(GameContainer gc, StateBasedGame sbg, Graphics grphcs) throws SlickException {
            grphcs.draw(map.getBanana().getYForX(g, beta, velocity, gc));
    }

    private void isEndPause(GameContainer gc, StateBasedGame sbg, int i) {
        Input input = gc.getInput();
        if (input.isKeyPressed(Input.KEY_P)) {
            currentState = statebeforpause;
        }
    }

    private void applyUserInput(GameContainer gc, StateBasedGame sbg, int i) {
        Input input = gc.getInput();

        /* read keyboard input */

        // check for key pressed
        if (!Settings.MOUSEENABLED) {
            if (input.isKeyDown(Input.KEY_LEFT)) {
                velocity -= 0.8f;
            } else if (input.isKeyDown(Input.KEY_RIGHT)) {
                velocity += 0.8f;
            } else if (input.isKeyDown(Input.KEY_UP)) {
                beta += 0.8f;
            } else if (input.isKeyDown(Input.KEY_DOWN)) {
                beta -= 0.8f;
            } // check for key down numpad
            else if (input.isKeyPressed(Input.KEY_NUMPAD4)) {
                velocity--;
            } else if (input.isKeyPressed(Input.KEY_NUMPAD6)) {
                velocity++;
            } else if (input.isKeyPressed(Input.KEY_NUMPAD8)) {
                beta++;
            } else if (input.isKeyPressed(Input.KEY_NUMPAD2)) {
                beta--;
            }
        }
        if (input.isKeyPressed(Input.KEY_P) || input.isKeyPressed(Input.KEY_ESCAPE)) {
            statebeforpause = currentState;
            currentState = STATES.PAUSE_GAME_STATE;
        } else if (input.isKeyPressed(Input.KEY_D)) {
            if (Settings.DEBUG) {
                Settings.DEBUG = false;
            } else {
                Settings.DEBUG = true;
            }
        }
        // check for spacebar pressed
        else if (input.isKeyPressed(Input.KEY_SPACE)) {
            throwBanana();
        }

        /* read mouse input */
        
        if (Settings.MOUSEENABLED) {
            beta = (float) Math.toDegrees(Math.atan((double) (((input.getMouseY() - map.getBanana().getSpawnPosition().y)) / (map.getBanana().getSpawnPosition().x - input.getMouseX()))));

            if (isPowerUp) {
                if (!input.isMouseButtonDown(Input.MOUSE_LEFT_BUTTON)) {
                    isPowerUp = false;
                    throwBanana();
                } else {
                    velocity += 0.8f;
                }

            } else if (input.isMouseButtonDown(Input.MOUSE_LEFT_BUTTON)) {
                velocity = MINVELOCITY;
                isPowerUp = true;
            }
        }

        // debug        
        if (Settings.DEBUG) {
            if (input.isKeyDown(Input.KEY_RCONTROL)) {
                map.getBanana().setPosition(new Vector2f(input.getMouseX(), input.getMouseY()));
                // banana hit ape
                if (collision()) {
                    currentState = STATES.GET_INPUT_STATE;
                    map.getBanana().resetBanana();
                }
                // banana hit wall
                if (hitWall(map.getBanana())) {
                    currentState = STATES.GET_INPUT_STATE;
                    map.getBanana().resetBanana();
                }
            }

        }

        checkAndCorrectInput();
    }

    /**
     * checks if the given values for velocity and angle are ok. if not, corrects them
     */
    private void checkAndCorrectInput() {
        if (velocity < MINVELOCITY) {
            velocity = MINVELOCITY;
        }
        if (velocity > MAXVELOCITY) {
            velocity = MAXVELOCITY;
        }
        if (beta > MAXBETA) {
            beta = MAXBETA;
        }
        if (beta < MINBETA) {
            beta = MINBETA;
        }

    }

    private void throwBanana() {
        time = 0.0f;
        float betaRadian = (float) Math.toRadians((double) beta);
        map.getBanana().startThrow(betaRadian, velocity);
        currentState = STATES.MOVING_PIECE_STATE;
    }

    private void calculateEnemyMovement(GameContainer gc, StateBasedGame sbg, int i) {
        deltaCounterEnemy -= i;

        if (deltaCounterEnemy < 0) {
            timeEnemy += 0.14040f;
            map.getBanana2().moveBanana(g, timeEnemy, gc);
            deltaCounterEnemy = MAXDELTACOUNTER;
        }

        // banana hit you
        if (collision2()) {
            currentState = STATES.GAME_OVER_STATE;
        }

        // banana hit wall
        if (hitWall(map.getBanana2())) {
            map.getBanana2().resetBanana();
        }
    }

    private void calculateMovement(GameContainer gc, StateBasedGame sbg, int i) {
        deltaCounter -= i;

        if (deltaCounter < 0) {
            time += 0.14040f;
            map.getBanana().moveBanana(g, time, gc);
            deltaCounter = MAXDELTACOUNTER;
        }

        if (!map.getBanana().isMoving()) {
            currentState = STATES.GET_INPUT_STATE;
        }

        // banana hit ape
        if (collision()) {            
            currentState = STATES.LEVEL_COMPLETE_STATE;
        }

        // banana hit wall
        if (hitWall(map.getBanana())) {
            currentState = STATES.GET_INPUT_STATE;
            map.getBanana().resetBanana();
        }
    }

    private boolean collision() {
        if (map.getBanana().isCollidingWith(map.getEnemy())) {
            return true;
        }
        return false;
    }

    private boolean collision2() {
        if (map.getBanana2().isCollidingWith(map.getApe())) {
            return true;
        }
        return false;
    }

    private boolean hitWall(Banana banana) {

        for (int i = 0; i < map.getEntities().size(); i++) {
            Block entity1 = (Block) map.getEntities().get(i);
            if (banana.isCollidingWith(entity1)) {
                if (Settings.MOUSEENABLED) {
                    velocity = MINVELOCITY;
                }
                return true;
            }
        }
        return false;
    }
}
