/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package apes;

import org.newdawn.slick.AngelCodeFont;
import org.newdawn.slick.Font;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.UnicodeFont;
import org.newdawn.slick.state.BasicGameState;
import org.newdawn.slick.state.StateBasedGame;

/**
 * the menu state of this game.
 * It displays options such as start game, exit, as well as sound, video
 * and input settings, which may be changed.
 *
 * @author sam
 */
public class MainMenuState extends BasicGameState {

    private static final String help = "Welcome!\n"
            + "\nControlls:"
            + "\nMouse:\nAngle: move mouse\nvelocity: hold left mouse button\nthrow: let go of left mouse button"
            + "\nKeyboard:\nUp: raise angle\nDown: lower angle\nRight: raise velocity\nLeft: lower velocity\n"
            + "Numbad: see above, but exactly one at a time\np: pause\n\nPress Enter to return to main menu.";
    //private Font font;
    
    // general setting
    private Image background = null;

    private int stateID = -1;

    private enum STATES {
        MAINMENU, OPTIONS, VIDEO, SOUND, INPUT, HELP, CHOOSELEVEL, GAME
    }
    private STATES currentState = null;

    private Menu main;
    private Menu options;
    private Menu game;
    private Menu video;
    private Menu sound;
    private Menu input;
    private Menu chooselevel;

    private SettingsLevel settingsLevel;

    public MainMenuState(int stateID) {
        this.stateID = stateID;
        settingsLevel = new SettingsLevel();
    }

    @Override
    public int getID() {
        return this.stateID;
    }

    @Override
    public void init(GameContainer gc, StateBasedGame sbg) throws SlickException {

        //font = new AngelCodeFont("/data/options/fonts/hawaianKiller.fnt", "/data/options/fonts/hawaianKiller_00.png");

        ((GameState) sbg.getState(Game.GAMEPLAYSTATE)).setSettingsLevel(settingsLevel);

        background = new Image("/data/gorilla2.gif");
        currentState = STATES.MAINMENU;        

        // load images
        Image startGameOption = new Image("/data/options/optionsStart.png");
        Image exitOption = new Image("/data/options/OptionsExit.png");
        Image optionOption = new Image("/data/options/optionsOptions.png");
        Image helpOption = new Image("/data/options/optionHelp.png");
        Image gameOption = new Image("/data/options/optionGame.png");

        Image inputOption = new Image("/data/options/optionsInput.png");
        Image soundOption = new Image("/data/options/optionsSound.png");
        Image backOption = new Image("/data/options/optionBack.png");
        Image videoOption = new Image("/data/options/optionVideo2.png");

        Image soundOff = new Image("/data/options/optionsOff.png");

        Image deviceMouse = new Image("/data/options/optionsDevideMouse.png");
        Image deviceKeyboard = new Image("/data/options/optionsDeviceKeyboard.png");

        Image fullscreenOn = new Image("/data/options/optionFullscreenYes.png");
        Image fullscreenOff = new Image("/data/options/optionFullscreenNo.png");

        Image[] lvl = new Image[settingsLevel.getNumberOfLevels()];
        for (int i = 0; i < lvl.length; i++) {
            lvl[i] = new Image("/data/options/lvl/lvl" + (i + 1) + ".png");

        }

        // set general x/y positions accordingly
        float centerXPosition = (gc.getScreenWidth() - Math.max(startGameOption.getWidth(), exitOption.getWidth())) / 2;
        float yspaceBetweenOptions = startGameOption.getHeight() / 2;
        float yPositionOfTopOption = gc.getScreenHeight() / 1.8f;

        // instanciate menus
        main = new Menu(centerXPosition, yPositionOfTopOption, yspaceBetweenOptions);
        game = new Menu(centerXPosition, yPositionOfTopOption, yspaceBetweenOptions);
        options = new Menu(centerXPosition, yPositionOfTopOption, yspaceBetweenOptions);
        video = new Menu(centerXPosition, yPositionOfTopOption, yspaceBetweenOptions);
        sound = new Menu(centerXPosition, yPositionOfTopOption, yspaceBetweenOptions);
        input =  new Menu(centerXPosition, yPositionOfTopOption, yspaceBetweenOptions);

        centerXPosition = (gc.getScreenWidth() - lvl[0].getWidth()) / 1.1f;
        yspaceBetweenOptions = lvl[0].getHeight() / 2;
        yPositionOfTopOption = gc.getScreenHeight() /6.4f;
        chooselevel = new Menu(centerXPosition, yPositionOfTopOption, yspaceBetweenOptions);

        // add items to main menu
        main.addItem(startGameOption, new MenuActionActivated() {
            @Override
            public void execute(GameContainer gc, StateBasedGame sbg, MenuItem item) throws SlickException {
                currentState = STATES.CHOOSELEVEL;
            } });
        main.addItem(exitOption, new MenuActionActivated() {
            @Override
            public void execute(GameContainer gc, StateBasedGame sbg, MenuItem item) throws SlickException {
                gc.exit();
            } });
        main.addItem(optionOption, new MenuActionActivated() {
            @Override
            public void execute(GameContainer gc, StateBasedGame sbg, MenuItem item) throws SlickException {
                currentState = STATES.OPTIONS;
            } });
        main.addItem(helpOption, new MenuActionActivated() {
            @Override
            public void execute(GameContainer gc, StateBasedGame sbg, MenuItem item) throws SlickException {
                currentState = STATES.HELP;
            } });

        // add items to level chooser
        for (int i = 0; i < settingsLevel.getNumberOfLevels(); i++) {
            final int currI = i + 1;
            chooselevel.addItem(lvl[i], new MenuAction() {

                @Override
                public void execute(GameContainer gc, StateBasedGame sbg, MenuItem item) throws SlickException {
                    ((GameState) sbg.getState(Game.GAMEPLAYSTATE)).setLevel("/data/lvl" + currI + ".tmx");
                    ((GameState) sbg.getState(Game.GAMEPLAYSTATE)).setLevel(currI);
                    //gc.reinit();
                    ((GameState) sbg.getState(Game.GAMEPLAYSTATE)).init(gc, sbg);
                    currentState = STATES.MAINMENU;
                    sbg.enterState(Game.GAMEPLAYSTATE);
                }

                @Override
                public boolean isActivated() {
                    return settingsLevel.isLevelOpen(currI);
                }
            }) ;
        }
        chooselevel.addItem(backOption, new MenuActionActivated() {
            @Override
            public void execute(GameContainer gc, StateBasedGame sbg, MenuItem item) throws SlickException {
                currentState = STATES.MAINMENU;
            } });

        // add items to options menu
        options.addItem(inputOption, new MenuActionActivated() {
            @Override
            public void execute(GameContainer gc, StateBasedGame sbg, MenuItem item) throws SlickException {
                currentState = STATES.INPUT;
            } });
        options.addItem(soundOption, new MenuActionActivated() {
            @Override
            public void execute(GameContainer gc, StateBasedGame sbg, MenuItem item) throws SlickException {
                currentState = STATES.SOUND;
            } });
        options.addItem(videoOption, new MenuActionActivated() {
            @Override
            public void execute(GameContainer gc, StateBasedGame sbg, MenuItem item) throws SlickException {
                currentState = STATES.VIDEO;
            } });
        options.addItem(gameOption, new MenuActionActivated() {
            @Override
            public void execute(GameContainer gc, StateBasedGame sbg, MenuItem item) throws SlickException {
                currentState = STATES.GAME;
            } });
        options.addItem(backOption, new MenuActionActivated() {
            @Override
            public void execute(GameContainer gc, StateBasedGame sbg, MenuItem item) throws SlickException {
                currentState = STATES.MAINMENU;
            } });

        // add items to game option
        Image aiChoosen = new Image("/data/options/optionsAISImple.png");
        if (Settings.AItoUse == 0) {
            aiChoosen = new Image("/data/options/optionsAISImple.png");
        } else if (Settings.AItoUse == 1) {
            aiChoosen = new Image("/data/options/optionsAIMedium1.png");
        } else if (Settings.AItoUse == 2) {
            aiChoosen = new Image("/data/options/optionsAIMedium2.png");
        } else if (Settings.AItoUse == 3) {
            aiChoosen = new Image("/data/options/optionsAIMedium3.png");
        }
        game.addItem(aiChoosen, new MenuActionActivated() {
            Image ai0 = new Image("/data/options/optionsAISImple.png");
            Image ai1 = new Image("/data/options/optionsAIMedium1.png");
            Image ai2 = new Image("/data/options/optionsAIMedium2.png");
            // Image ai3 = new Image("/data/options/optionsAIMedium3.png");
            int selected = Settings.AItoUse;
            @Override
            public void execute(GameContainer gc, StateBasedGame sbg, MenuItem item) throws SlickException {
                if (selected == 0) {
                    selected = 1;
                    item.setImg(ai1);
                    Settings.AItoUse = 1;
                } else if (selected == 1) {
                    selected = 2;
                    item.setImg(ai2);
                    Settings.AItoUse = 2;
                } else if (selected == 2) {
                    selected = 0;
                    item.setImg(ai0);
                    Settings.AItoUse = 0;
                    /*selected = 3;
                    item.setImg(ai3);
                    Settings.AItoUse = 3;*/
                }/*} else if (selected == 3) {
                    selected = 0;
                    item.setImg(ai0);
                    Settings.AItoUse = 0;
                }*/
            }
        });
        game.addItem(backOption, new MenuActionActivated() {
            @Override
            public void execute(GameContainer gc, StateBasedGame sbg, MenuItem item) throws SlickException {
                currentState = STATES.OPTIONS;
            } });

        // add items to sound option
        sound.addItem(soundOff, new MenuAction() {
            @Override
            public void execute(GameContainer gc, StateBasedGame sbg, MenuItem item) throws SlickException {
                    // TODO if sound implemented, insert action to turn it on
            }
            @Override
            public boolean isActivated() {
                return false;
            }

        });
        sound.addItem(backOption, new MenuActionActivated() {
            @Override
            public void execute(GameContainer gc, StateBasedGame sbg, MenuItem item) throws SlickException {
                currentState = STATES.OPTIONS;
            } });

        // add items to input option
        Image deviceChosen;
        if (Settings.MOUSEENABLED) {
            deviceChosen = deviceMouse;
        } else {
            deviceChosen = deviceKeyboard;
        }
        input.addItem(deviceChosen, new MenuActionActivated() {
            Image deviceMouse = new Image("/data/options/optionsDevideMouse.png");
            Image deviceKeyboard = new Image("/data/options/optionsDeviceKeyboard.png");
            int selected = 0;
            @Override
            public void execute(GameContainer gc, StateBasedGame sbg, MenuItem item) throws SlickException {
                if (selected == 0) {
                    selected = 1;
                    item.setImg(deviceMouse);
                    Settings.MOUSEENABLED = true;
                } else {
                    selected = 0;
                    item.setImg(deviceKeyboard);
                    Settings.MOUSEENABLED = false;
                }
            }
        });
        input.addItem(backOption, new MenuActionActivated() {
            @Override
            public void execute(GameContainer gc, StateBasedGame sbg, MenuItem item) throws SlickException {
                currentState = STATES.OPTIONS;
            } });

        // add items to video option
        Image fullscreenChosen;
        if (gc.isFullscreen()) {
            fullscreenChosen = fullscreenOn;
        } else {
            fullscreenChosen = fullscreenOff;
        }
        video.addItem(fullscreenChosen, new MenuActionActivated() {
        Image fullscreenOn = new Image("/data/options/optionFullscreenYes.png");
        Image fullscreenOff = new Image("/data/options/optionFullscreenNo.png");
            int selected = 0;
            @Override
            public void execute(GameContainer gc, StateBasedGame sbg, MenuItem item) throws SlickException {
                if (selected == 0) {
                    selected = 1;
                    item.setImg(fullscreenOff);
                    gc.setFullscreen(false);
                    //Game.app.setDisplayMode(800, 600, false);
                } else {
                    selected = 0;
                    item.setImg(fullscreenOn);
                    //Game.app.setDisplayMode(1280, 960, true);
                    gc.setFullscreen(true);
                }
            }
        });
        video.addItem(backOption, new MenuActionActivated() {
            @Override
            public void execute(GameContainer gc, StateBasedGame sbg, MenuItem item) throws SlickException {
                currentState = STATES.OPTIONS;
            } });
    }

    @Override
    public void render(GameContainer gc, StateBasedGame sbg, Graphics grphcs) throws SlickException {

        // render the background
        background.drawCentered((gc.getScreenWidth() / 2), (gc.getScreenHeight() / 2));

        switch (currentState) {
            case MAINMENU:
                main.render(grphcs);
                break;
            case OPTIONS:
                options.render(grphcs);
                break;
            case VIDEO:
                video.render(grphcs);
                break;
            case SOUND:
                sound.render(grphcs);
                break;
            case INPUT:
                input.render(grphcs);
                break;
            case HELP:
               // font.drawString(gc.getScreenWidth() / 16, gc.getScreenHeight() / 16, help);
                grphcs.drawString(help, gc.getScreenWidth() / 16, gc.getScreenHeight() / 16);
                break;
            case CHOOSELEVEL:
                chooselevel.render(grphcs);
                break;
            case GAME:
                game.render(grphcs);
                break;
        }
    }

    @Override
    public void update(GameContainer gc, StateBasedGame sbg, int i) throws SlickException {

        switch (currentState) {
            case MAINMENU:
                main.update(gc, sbg, i);
                break;
            case OPTIONS:
                options.update(gc, sbg, i);
                break;
            case VIDEO:
                video.update(gc, sbg, i);
                break;
            case SOUND:
                sound.update(gc, sbg, i);
                break;
            case INPUT:
                input.update(gc, sbg, i);
                break;
            case HELP:
                Input input = gc.getInput();
                if (input.isKeyPressed(Input.KEY_ENTER)) {
                    currentState = STATES.MAINMENU;
                }
                break;
            case CHOOSELEVEL:
                chooselevel.update(gc, sbg, i);
                break;
            case GAME:
                game.update(gc, sbg, i);
                break;
        }
    }
}
