/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package apes;

import java.util.LinkedList;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.StateBasedGame;

/**
 * This class represents a Menu.
 * In this case, Menu means one container containing one or more MenuItems,
 * which themselves can link to a different Menu or be responsible for a setting
 * or something similar.
 *
 * @see MenuItem
 *
 * @author sam
 */
public class Menu {

    private LinkedList<MenuItem> items;

    private float centerXPosition;
    private float topYPosition;
    private float yspaceBetweenItems;

    /**
     * constructs a new Menu.
     *
     * @param centerXPosition
     * @param topYPosition
     * @param yspaceBetweenItems
     */
    // TODO check if centerXPosition really does what it says. (hint: it prop. does not)
    public Menu(float centerXPosition, float topYPosition, float yspaceBetweenItems) {
        this.centerXPosition = centerXPosition;
        this.topYPosition = topYPosition;
        this.yspaceBetweenItems = yspaceBetweenItems;
        items = new LinkedList<MenuItem>();
    }

    /**
     * add a new MenuItem to this menu.
     *
     * @param img the image to represent this item
     * @param action the action to be executed by this item
     * @throws SlickException
     */
    public void addItem(Image img, MenuAction action) {

        MenuItem newItem = new MenuItem(img,
                centerXPosition, topYPosition + (items.size() * yspaceBetweenItems), yspaceBetweenItems / 2,
                action);
        items.add(newItem);
    }

    /**
     * let this menu or its MenuItems react to userinput/changed conditions.
     *
     * @param gc
     * @param sbg
     * @param i
     * @throws SlickException
     */
    public void update(GameContainer gc, StateBasedGame sbg, int i) throws SlickException {
        Input input = gc.getInput();

        int mouseX = input.getMouseX();
        int mouseY = input.getMouseY();

        boolean mouseClicked = input.isMousePressed(Input.MOUSE_LEFT_BUTTON);

        for (int j = 0; j < items.size(); j++) {
            MenuItem menuItem = items.get(j);
            menuItem.applyMouseInput(mouseX, mouseY, mouseClicked, i, gc, sbg);
        }
    }

    /**
     * draws this menu/its MenuItems to the screen.
     *
     * @param grphcs
     */
    public void render(Graphics grphcs) {
        for (int i = 0; i < items.size(); i++) {
            MenuItem menuItem = items.get(i);
            menuItem.render();            
            if (Settings.DEBUG) { grphcs.draw(menuItem.getPolygon()); }
        }
    }

}
